@@ -8,20 +8,20 @@
 static VALUE cMeasureAllocations;
 VALUE total_allocated_objects_key;
 
-static double measure_allocations_via_gc_stats(rb_trace_arg_t* trace_arg)
-{
-    return rb_gc_stat(total_allocated_objects_key);
-}
-
-static double measure_allocations_via_tracing(rb_trace_arg_t* trace_arg)
+static double measure_allocations(rb_trace_arg_t* trace_arg)
 {
     static double result = 0;
 
     if (trace_arg)
     {
+        // Only process creation of new objects
         rb_event_flag_t event = rb_tracearg_event_flag(trace_arg);
-        if (event == RUBY_INTERNAL_EVENT_NEWOBJ)
-            result++;
+        if (event == RUBY_INTERNAL_EVENT_NEWOBJ) {
+            // Don't count allocations of internal IMemo objects
+            VALUE object = rb_tracearg_object(trace_arg);
+            if (BUILTIN_TYPE(object) != T_IMEMO)
+                result++;
+        }
     }
     return result;
 }
@@ -30,14 +30,11 @@ prof_measurer_t* prof_measurer_allocations(bool track_allocations)
 {
     prof_measurer_t* measure = ALLOC(prof_measurer_t);
     measure->mode = MEASURE_ALLOCATIONS;
+    measure->measure = measure_allocations;
     measure->multiplier = 1;
+    // Need to track allocations to get RUBY_INTERNAL_EVENT_NEWOBJ event
     measure->track_allocations = track_allocations;
 
-    if (track_allocations)
-        measure->measure = measure_allocations_via_tracing;
-    else
-        measure->measure = measure_allocations_via_gc_stats;
-
     return measure;
 }
 
