@@ -7,20 +7,23 @@
 
 static VALUE cMeasureMemory;
 
-static double
-measure_memory_via_tracing(rb_trace_arg_t* trace_arg)
+static double measure_memory(rb_trace_arg_t* trace_arg)
 {
     static double result = 0;
 
     if (trace_arg)
     {
+        // Only process creation of new objects
         rb_event_flag_t event = rb_tracearg_event_flag(trace_arg);
         if (event == RUBY_INTERNAL_EVENT_NEWOBJ)
         {
+            // Don't count allocations of internal IMemo objects
             VALUE object = rb_tracearg_object(trace_arg);
-            result += rb_obj_memsize_of(object);
+            if (BUILTIN_TYPE(object) != T_IMEMO)
+                result += rb_obj_memsize_of(object);
         }
     }
+
     return result;
 }
 
@@ -28,8 +31,9 @@ prof_measurer_t* prof_measurer_memory(bool track_allocations)
 {
   prof_measurer_t* measure = ALLOC(prof_measurer_t);
   measure->mode = MEASURE_MEMORY;
-  measure->measure = measure_memory_via_tracing;
+  measure->measure = measure_memory;
   measure->multiplier = 1;
+  // Need to track allocations to get RUBY_INTERNAL_EVENT_NEWOBJ event
   measure->track_allocations = true;
   return measure;
 }
