@@ -36,7 +36,7 @@ typedef struct prof_measurement_t
     VALUE object;
 } prof_measurement_t;
 
-prof_measurer_t* prof_get_measurer(prof_measure_mode_t measure, bool track_allocations);
+prof_measurer_t* prof_measurer_create(prof_measure_mode_t measure, bool track_allocations);
 double prof_measure(prof_measurer_t* measurer, rb_trace_arg_t* trace_arg);
 
 prof_measurement_t* prof_measurement_create(void);
