@@ -211,7 +211,7 @@ static void prof_event_hook(VALUE trace_point, void* data)
     }
 
     /* Special case - skip any methods from the mProf
-     module since they clutter the results but aren't important to them results. */
+     module since they clutter the results but aren't important. */
     if (self == mProf)
         return;
 
@@ -226,6 +226,8 @@ static void prof_event_hook(VALUE trace_point, void* data)
         {
             prof_frame_t* frame = prof_frame_current(thread_data->stack);
 
+            /* If there is no frame then this is either the first method being profiled or we have climbed the
+               call stack higher than where we started. */
             if (!frame)
             {
                 prof_method_t* method = check_method(profile, trace_arg, event, thread_data);
@@ -236,11 +238,13 @@ static void prof_event_hook(VALUE trace_point, void* data)
                 prof_call_tree_t* call_tree = prof_call_tree_create(method, NULL, method->source_file, method->source_line);
                 prof_add_call_tree(method->call_trees, call_tree);
 
+                // We have climbed higher in the stack then where we started
                 if (thread_data->call_tree)
                 {
                     prof_call_tree_add_parent(thread_data->call_tree, call_tree);
                     frame = prof_frame_unshift(thread_data->stack, call_tree, thread_data->call_tree, measurement);
                 }
+                // This is the first method to be profiled
                 else
                 {
                     frame = prof_frame_push(thread_data->stack, call_tree, measurement, RTEST(profile_t->paused));
@@ -374,7 +378,7 @@ prof_profile_t* prof_get_profile(VALUE self)
 static int collect_threads(st_data_t key, st_data_t value, st_data_t result)
 {
     thread_data_t* thread_data = (thread_data_t*)value;
-    if (thread_data->trace)
+    if (thread_data->trace && thread_data->call_tree)
     {
         VALUE threads_array = (VALUE)result;
         rb_ary_push(threads_array, prof_thread_wrap(thread_data));
@@ -565,7 +569,7 @@ static VALUE prof_initialize(int argc, VALUE* argv, VALUE self)
     {
         Check_Type(mode, T_FIXNUM);
     }
-    profile->measurer = prof_get_measurer(NUM2INT(mode), track_allocations == Qtrue);
+    profile->measurer = prof_measurer_create(NUM2INT(mode), track_allocations == Qtrue);
     profile->allow_exceptions = (allow_exceptions == Qtrue);
 
     if (exclude_threads != Qnil)
@@ -854,8 +858,14 @@ static VALUE prof_exclude_method(VALUE self, VALUE klass, VALUE msym)
 /* :nodoc: */
 VALUE prof_profile_dump(VALUE self)
 {
+    prof_profile_t* profile = prof_get_profile(self);
+  
     VALUE result = rb_hash_new();
     rb_hash_aset(result, ID2SYM(rb_intern("threads")), prof_threads(self));
+    rb_hash_aset(result, ID2SYM(rb_intern("measurer_mode")), INT2NUM(profile->measurer->mode));
+    rb_hash_aset(result, ID2SYM(rb_intern("measurer_track_allocations")), 
+                 profile->measurer->track_allocations ? Qtrue : Qfalse);
+
     return result;
 }
 
@@ -864,6 +874,11 @@ VALUE prof_profile_load(VALUE self, VALUE data)
 {
     prof_profile_t* profile = prof_get_profile(self);
 
+    VALUE measurer_mode = rb_hash_aref(data, ID2SYM(rb_intern("measurer_mode")));
+    VALUE measurer_track_allocations = rb_hash_aref(data, ID2SYM(rb_intern("measurer_track_allocations")));
+    profile->measurer = prof_measurer_create((prof_measure_mode_t)(NUM2INT(measurer_mode)),
+                                              measurer_track_allocations == Qtrue ? true : false);
+
     VALUE threads = rb_hash_aref(data, ID2SYM(rb_intern("threads")));
     for (int i = 0; i < rb_array_len(threads); i++)
     {
