@@ -2,8 +2,8 @@
 # encoding: UTF-8
 
 require File.expand_path('../test_helper', __FILE__)
-require 'stringio'
 require 'fileutils'
+require 'stringio'
 require 'tmpdir'
 require_relative 'prime'
 
@@ -37,10 +37,10 @@ class PrintersTest < TestCase
   end
 
   def test_print_to_files
-    output_dir = 'examples2'
+    output_dir = 'tmp/examples2'
 
     if ENV['SAVE_NEW_PRINTER_EXAMPLES']
-      output_dir = 'examples'
+      output_dir = 'tmp/examples'
     end
     FileUtils.mkdir_p output_dir
 
